'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _babelTypes = require('babel-types');

var t = _interopRequireWildcard(_babelTypes);

var _babelPluginSyntaxFlow = require('babel-plugin-syntax-flow');

var _babelPluginSyntaxFlow2 = _interopRequireDefault(_babelPluginSyntaxFlow);

var _babelLooksLike = require('babel-looks-like');

var _babelLooksLike2 = _interopRequireDefault(_babelLooksLike);

var _babylon = require('babylon');

var _babelTraverse = require('babel-traverse');

var _babelTraverse2 = _interopRequireDefault(_babelTraverse);

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _constantCase = require('constant-case');

var _constantCase2 = _interopRequireDefault(_constantCase);

var _s2sUtils = require('s2s-utils');

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// import blog from 'babel-log'

const builders = {
  test: (0, _s2sUtils.template)(`test(NAME, () => {
    expect(actions.FUNC()).toMatchSnapshot();
  });`)
};


const createTest = name => builders.test({
  NAME: t.stringLiteral((0, _constantCase2.default)(name)),
  FUNC: t.identifier(_lodash2.default.camelCase(name))
});

exports.default = () => ({
  inherits: _babelPluginSyntaxFlow2.default,
  name: 's2s-d-action-tests',
  visitor: {
    Program(rootPath, {
      opts: { from }
    }) {
      if (!from) {
        throw new Error('required from option');
      }

      const existTestCases = [];
      rootPath.traverse({
        CallExpression(callPath) {
          if ((0, _babelLooksLike2.default)(callPath, {
            node: {
              callee: { type: 'Identifier', name: 'test' }
            }
          })) {
            const testTitlePath = callPath.get('arguments')[0];
            existTestCases.push(testTitlePath.get('value').node);
          }
        }
      });

      const code = _fs2.default.readFileSync(from, 'utf8');
      const ast = (0, _babylon.parse)(code, {
        sourceType: 'module',
        plugins: ['flow', 'objectRestSpread']
      });

      const typeNameSet = new Set();

      (0, _babelTraverse2.default)(ast, {
        TypeAlias(path) {
          if (path.get('id').node.name === 'Action') {
            const right = path.get('right');
            if (right.isUnionTypeAnnotation()) {
              for (const typePath of right.get('types')) {
                typeNameSet.add(typePath.get('id').node.name);
              }
            } else if (right.isGenericTypeAnnotation()) {
              typeNameSet.add(right.get('id').node.name);
            }
            // remove `type Action = ...`
            path.remove();
          }
        }
      });

      function add(actionType) {
        rootPath.pushContainer('body', [t.noop(), createTest(actionType)]);
      }

      const actions = [...typeNameSet];
      actions.map(v => (0, _constantCase2.default)(v)).filter(v => !existTestCases.includes(v)).forEach(v => add(v));
    }
  }
});